@@ -62,7 +62,7 @@ mod tests {
     use quickcheck_macros::quickcheck;
 
     use crate::tests::{NonZeroTinyInt, TinyInt};
-    use crate::{Mint, MintRequest, MintTransaction};
+    use crate::{KeyManager, Mint, ReissueRequest, ReissueTransaction};
 
     fn divide(amount: u64, n_ways: u8) -> impl Iterator<Item = u64> {
         (0..n_ways).into_iter().map(move |i| {
@@ -79,7 +79,7 @@ mod tests {
         dbc: &Dbc,
         n_ways: u8,
         output_owner: &threshold_crypto::PublicKeySet,
-    ) -> MintRequest {
+    ) -> ReissueRequest {
         let inputs = HashSet::from_iter(vec![dbc.clone()]);
         let input_hashes = BTreeSet::from_iter(inputs.iter().map(|in_dbc| in_dbc.name()));
 
@@ -93,7 +93,7 @@ mod tests {
                 )
             }));
 
-        let transaction = MintTransaction { inputs, outputs };
+        let transaction = ReissueTransaction { inputs, outputs };
 
         let sig_share = dbc_owner
             .secret_key_share
@@ -104,7 +104,7 @@ mod tests {
             .combine_signatures(vec![(0, &sig_share)])
             .unwrap();
 
-        MintRequest {
+        ReissueRequest {
             transaction,
             input_ownership_proofs: HashMap::from_iter(vec![(
                 dbc.name(),
@@ -116,7 +116,7 @@ mod tests {
     #[test]
     fn test_dbc_without_inputs_is_invalid() {
         let input_content = DbcContent::new(
-            Default::default(),
+            BTreeSet::new(),
             100,
             0,
             crate::bls_dkg_id().public_key_set.public_key(),
@@ -127,7 +127,7 @@ mod tests {
         let dbc = Dbc {
             content: input_content,
             transaction: DbcTransaction {
-                inputs: Default::default(),
+                inputs: BTreeSet::new(),
                 outputs: input_content_hashes,
             },
             transaction_sigs: Default::default(),
@@ -153,32 +153,65 @@ mod tests {
     ) {
         let amount = 100;
         let genesis_owner = crate::bls_dkg_id();
-        let (mut genesis, genesis_dbc) =
-            Mint::genesis(genesis_owner.public_key_set.clone(), amount);
+        let genesis_key = genesis_owner.public_key_set.public_key();
+
+        let mut genesis_node = Mint::new(KeyManager::new(
+            genesis_owner.public_key_set.clone(),
+            (0, genesis_owner.secret_key_share.clone()),
+            genesis_key,
+        ));
+
+        let (gen_dbc_content, gen_dbc_trans, (gen_key_set, gen_node_sig)) =
+            genesis_node.issue_genesis_dbc(amount).unwrap();
+
+        let genesis_sig = gen_key_set
+            .combine_signatures(vec![gen_node_sig.threshold_crypto()])
+            .unwrap();
+
+        let genesis_dbc = Dbc {
+            content: gen_dbc_content,
+            transaction: gen_dbc_trans,
+            transaction_sigs: BTreeMap::from_iter(vec![(
+                crate::mint::GENESIS_DBC_INPUT,
+                (genesis_key, genesis_sig),
+            )]),
+        };
 
         let input_owner = crate::bls_dkg_id();
-        let mint_request = prepare_even_split(
+        let reissue_request = prepare_even_split(
             &genesis_owner,
             &genesis_dbc,
             n_inputs.coerce(),
             &input_owner.public_key_set,
         );
-        let input_hashes = mint_request
+        let input_hashes = reissue_request
             .transaction
             .inputs
             .iter()
             .map(|i| i.name())
             .collect();
-        let (split_transaction, split_transaction_sigs) =
-            genesis.reissue(mint_request.clone(), input_hashes).unwrap();
+        let (split_transaction, split_transaction_sigs) = genesis_node
+            .reissue(reissue_request.clone(), input_hashes)
+            .unwrap();
 
-        assert_eq!(split_transaction, mint_request.transaction.blinded());
+        assert_eq!(split_transaction, reissue_request.transaction.blinded());
 
-        let inputs = HashSet::from_iter(mint_request.transaction.outputs.into_iter().map(
-            |content| Dbc {
-                content,
-                transaction: split_transaction.clone(),
-                transaction_sigs: split_transaction_sigs.clone(),
+        let (mint_key_set, mint_sig_share) = split_transaction_sigs.values().next().unwrap();
+        let mint_sig = mint_key_set
+            .combine_signatures(vec![mint_sig_share.threshold_crypto()])
+            .unwrap();
+
+        let inputs = HashSet::from_iter(reissue_request.transaction.outputs.into_iter().map(
+            |content| {
+                Dbc {
+                    content,
+                    transaction: split_transaction.clone(),
+                    transaction_sigs: BTreeMap::from_iter(
+                        split_transaction_sigs
+                            .iter()
+                            .map(|(input, _)| (*input, (genesis_key, mint_sig.clone()))),
+                    ),
+                }
             },
         ));
 
@@ -192,7 +225,7 @@ mod tests {
         );
         let outputs = HashSet::from_iter(vec![content]);
 
-        let transaction = MintTransaction { inputs, outputs };
+        let transaction = ReissueTransaction { inputs, outputs };
         let sig_share = input_owner
             .secret_key_share
             .sign(&transaction.blinded().hash());
@@ -209,15 +242,20 @@ mod tests {
             )
         }));
 
-        let mint_request = MintRequest {
+        let reissue_request = ReissueRequest {
             transaction,
             input_ownership_proofs,
         };
 
-        let (transaction, transaction_sigs) = genesis
-            .reissue(mint_request.clone(), input_hashes.clone())
+        let (transaction, transaction_sigs) = genesis_node
+            .reissue(reissue_request.clone(), input_hashes.clone())
+            .unwrap();
+        assert_eq!(reissue_request.transaction.blinded(), transaction);
+
+        let (mint_key_set, mint_sig_share) = transaction_sigs.values().next().unwrap();
+        let mint_sig = mint_key_set
+            .combine_signatures(vec![mint_sig_share.threshold_crypto()])
             .unwrap();
-        assert_eq!(mint_request.transaction.blinded(), transaction);
 
         let fuzzed_parents = BTreeSet::from_iter(
             input_hashes
@@ -238,16 +276,16 @@ mod tests {
             crate::bls_dkg_id().public_key_set.public_key(),
         );
 
-        let mut fuzzed_transaction_sigs = BTreeMap::new();
+        let mut fuzzed_transaction_sigs: BTreeMap<Hash, (PublicKey, Signature)> = BTreeMap::new();
 
         // Add valid sigs
         fuzzed_transaction_sigs.extend(
             transaction_sigs
                 .iter()
                 .take(n_valid_sigs.coerce())
-                .to_owned(),
+                .map(|(in_hash, _)| (*in_hash, (genesis_key, mint_sig.clone()))),
         );
-        let mut repeating_inputs = mint_request
+        let mut repeating_inputs = reissue_request
             .transaction
             .inputs
             .iter()
@@ -257,32 +295,40 @@ mod tests {
 
         // Valid mint signatures BUT signing wrong message
         for _ in 0..n_wrong_signer_sigs.coerce() {
-            use crate::key_manager::{ed25519_keypair, PublicKey, Signature};
-            use ed25519::Signer;
-
             if let Some(input) = repeating_inputs.next() {
-                let keypair = ed25519_keypair();
-                let transaction_sig = keypair.sign(&transaction.hash());
-                fuzzed_transaction_sigs.insert(
-                    input.name(),
-                    (PublicKey(keypair.public), Signature(transaction_sig)),
+                let id = crate::bls_dkg_id();
+                let key_mgr = KeyManager::new(
+                    id.public_key_set.clone(),
+                    (0, id.secret_key_share),
+                    genesis_key,
                 );
+                let trans_sig_share = key_mgr.sign(&transaction.hash());
+                let trans_sig = id
+                    .public_key_set
+                    .combine_signatures(vec![trans_sig_share.threshold_crypto()])
+                    .unwrap();
+                fuzzed_transaction_sigs
+                    .insert(input.name(), (id.public_key_set.public_key(), trans_sig));
             }
         }
 
         // Valid mint signatures BUT signing wrong message
         for _ in 0..n_wrong_msg_sigs.coerce() {
             if let Some(input) = repeating_inputs.next() {
-                let wrong_msg_sig = genesis.key_mgr.sign(&Hash([0u8; 32]));
-                fuzzed_transaction_sigs.insert(input.name(), (genesis.public_key(), wrong_msg_sig));
+                let wrong_msg_sig = genesis_node.key_mgr.sign(&Hash([0u8; 32]));
+                let wrong_msg_mint_sig = genesis_node
+                    .key_mgr
+                    .public_key_set()
+                    .combine_signatures(vec![wrong_msg_sig.threshold_crypto()])
+                    .unwrap();
+
+                fuzzed_transaction_sigs.insert(input.name(), (genesis_key, wrong_msg_mint_sig));
             }
         }
 
         // Valid mint signatures for inputs not present in the transaction
-        if let Some((_, key_sig)) = transaction_sigs.iter().next() {
-            for _ in 0..n_extra_input_sigs.coerce() {
-                fuzzed_transaction_sigs.insert(rand::random(), key_sig.to_owned());
-            }
+        for _ in 0..n_extra_input_sigs.coerce() {
+            fuzzed_transaction_sigs.insert(rand::random(), (genesis_key, mint_sig.clone()));
         }
 
         let dbc = Dbc {
@@ -291,7 +337,7 @@ mod tests {
             transaction_sigs: fuzzed_transaction_sigs,
         };
 
-        let key_cache = KeyCache::from(vec![genesis.public_key()]);
+        let key_cache = KeyCache::from(vec![genesis_key]);
         let validation_res = dbc.confirm_valid(&key_cache);
 
         println!("Validation Result: {:#?}", validation_res);
@@ -311,9 +357,6 @@ mod tests {
             Err(Error::MissingSignatureForInput) => {
                 assert!(n_valid_sigs.coerce::<u8>() < n_inputs.coerce::<u8>());
             }
-            Err(Error::Ed25519(_)) => {
-                assert!(n_wrong_msg_sigs.coerce::<u8>() > 0);
-            }
             Err(Error::UnknownInput) => {
                 assert!(n_extra_input_sigs.coerce::<u8>() > 0);
                 assert_ne!(
@@ -341,6 +384,9 @@ mod tests {
             Err(Error::TransactionMustHaveAnInput) => {
                 assert_eq!(n_inputs.coerce::<u8>(), 0);
             }
+            Err(Error::FailedSignature) => {
+                assert_ne!(n_wrong_msg_sigs.coerce::<u8>(), 0);
+            }
             res => panic!("Unexpected verification result {:?}", res),
         }
     }
