@@ -27,8 +27,8 @@ pub use crate::{
     dbc_content::{BlindedOwner, DbcContent},
     dbc_transaction::DbcTransaction,
     error::{Error, Result},
-    key_manager::{ChainNode, KeyCache, KeyManager, PublicKey, Signature},
-    mint::{Mint, MintRequest, MintTransaction},
+    key_manager::{KeyCache, KeyManager, NodeSignature, PublicKey, PublicKeySet, Signature},
+    mint::{Mint, ReissueRequest, ReissueTransaction},
 };
 
 impl From<[u8; 32]> for Hash {
